/*
 * File    : main.c
 * Purpose : main function
 */

#include <stdlib.h>
#include <avr/io.h>
#include <string.h>
#include <avr/interrupt.h>

#include "comm.h"
#include "delay.h"
#include "lcd.h"
#include "led.h"

#define BTN_TIMEOUT  25000

extern bool comm_use_red_led;
extern bool comm_use_green_led;

static char dec2hex[16] = "0123456789abcdef";

/*
 * convert int to string as hex
 */
void
to_hex ( uchar b, char * buf )
{
    buf[0] = '0';
    buf[1] = 'x';
    buf[2] = dec2hex[ (b / 16) ]; b = b % 16;
    buf[3] = dec2hex[ b ];
    buf[4] = '\0';
}

/*
 * convert int to string as binary
 */
void
to_bin ( uchar b, char * buf )
{
    uchar  i;

    for ( i = 0; i < 8; i++ )
    {
        if ( b & (1 << i) ) buf[7-i] = '1';
        else                buf[7-i] = '0';
    }

    buf[8] = '\0';
}

/*
 * eval command from string
 */
void
eval ( char * msg )
{
    uchar  i, x, y;
    char   buf[8];
    
    
    if ( strlen(msg) < 3 )
    {
        lcd_clear();
        lcd_puts( "ERR CMD: " );
        
        for ( i = 0; (msg[i] != '\0') && (i < 10); i++ )
            lcd_putc( msg[i] );
    }
    
    if ( msg[1] != '=' )
    {
        lcd_clear();
        lcd_puts( "ERR FMT:" );
        
        for ( i = 0; (msg[i] != '\0') && (i < 10); i++ )
            lcd_putc( msg[i] );
    }
    
    switch ( msg[0] )
    {
    case 'd' : /* write to display */
        lcd_puts( msg + 2 );
        break;
        
    case 'g' : /* goto xy */
        buf[0] = msg[2];
        buf[1] = msg[3];
        buf[2] = '\0';
        x = atoi( buf );
        buf[0] = msg[4];
        buf[1] = msg[5];
        buf[2] = '\0';
        y = atoi( buf );
        lcd_gotoxy( x, y );
        break;
        
    case 'c' : /* command */
        switch ( msg[2] )
        {
        case 'c' : /* clear display */
            lcd_clear();
            break;
            
        case 'n' : /* normal cursor */
            lcd_cursor_on();
            break;
            
        case 'i' : /* invisible cursor */
            lcd_cursor_off();
            break;
            
        case 'b' : /* blink cursor */
            lcd_cursor_blink();
            break;
            
        case 's' : /* go to sleep mode */
            sbi( MCUCR, 7 );
            break;
        }
        break;
        
    case 'l' : /* control communication leds */
        switch ( msg[2] )
        {
        case 'g' :
            comm_use_green_led = (msg[3] == '0' ? 0 : 1);
            led_off( LED_GREEN );
            break;
            
        case 'r' :
            comm_use_red_led = (msg[3] == '0' ? 0 : 1);
            led_off( LED_RED );
            break;
        }
        break;

    case 'p' : /* program characters */
        switch ( msg[2] )
        {
        case 'h' : /* horizontal bars */
            lcd_hbars();
            break;

        case 'v' : /* vertical bars */
            lcd_vbars();
            break;
        }
        break;
        
    case 'v' : /* show version */
        lcd_clear();
        lcd_puts( "AVRLCD v0.2, by" );
        lcd_gotoxy( 0, 1 );
        lcd_puts( "Ronald Kriemann" );
        break;
        
    default: /* unknown command */
        lcd_clear();
        lcd_puts( "ERR CMD:" );
        
        for ( i = 0; (msg[i] != '\0') && (i < 10); i++ )
            lcd_putc( msg[i] );
    }
}

/*
 * main function
 */
int
main( void )
{
    uchar  i;
    uint   ign_btn[5] = { 0, 0, 0, 0, 0 };
    char   buf[32];

    /* globally enable interrupts */
    sei();
    
    led_init();
    
    comm_init();
        
    lcd_init();
    lcd_clear();
    
    lcd_clear();
    lcd_puts( "LCD v0.2, by" );
    lcd_gotoxy( 0, 1 );
    lcd_puts( "Ronald Kriemann" );
    delay_ms( 500 );
    lcd_clear();
    
    /* init buttons */
    outb( PORTD, 0x00 );
    outb( DDRD, 0x00 );
    outb( PORTD, 0x00 );

#if 0
    /* for timing */
    while ( 1 )
    {
        led_on( LED_GREEN );
        delay_ms( 1000 );
        led_off( LED_GREEN );
        delay_ms( 1000 );
    }
#endif
    
    while ( 1 )
    {
        /*
         * receive command
         */
        
        if ( comm_has_rx() )
        {
            if ( comm_gets( buf, sizeof(buf) ) > 0 )
                eval( buf );
        }

        /*
         * check button status
         */

        for ( i = 0; i < 5; i++ )
        {
            if ( ! ign_btn[i] && bit_is_clear( PIND, i ) )
            {
                ign_btn[i] = BTN_TIMEOUT;
                comm_puts( "b=" );
                comm_putc( '0' + i );
                comm_putc( '\n' );
            }
        }
        
        /* reduce button timeout */
        for ( i = 0; i < 5; i++ )
        {
            if ( ign_btn[i] > 0 )
                ign_btn[i]--;
        }
    }
    
    return 0;
}

